/*
 * Include header files for all drivers that have been imported from
 * Atmel Software Framework (ASF).
 */
/*
 * Support and FAQ: visit <a href="https://www.microchip.com/support/">Microchip Support</a>
 */
#include <asf.h>


#define PIN_DATA						EXT1_PIN_13		//PB0


struct waveconfig_t
{
	uint32_t clock_sel;		// Internal clock signals selection
	uint32_t frequency;		//Waveform frequency (in Hz)
	uint32_t dutycycle;		//Duty cycle in percent (positive)
};


/** TC waveform configurations */
static const struct waveconfig_t s_tc0_ch0_waveconfig = {TC_CMR_TCCLKS_TIMER_CLOCK2, 1000000, 50};
static const uint32_t freq_divisors[5] = { 2, 8, 32, 128, 0};
static volatile uint32_t s_tc0_count = 0;




/**
 *  Configure UART console.
 */
static void configure_console(void)
{
	const usart_serial_options_t uart_serial_options = {
		.baudrate = CONF_UART_BAUDRATE,
#ifdef CONF_UART_CHAR_LENGTH
		.charlength = CONF_UART_CHAR_LENGTH,
#endif
		.paritytype = CONF_UART_PARITY,
#ifdef CONF_UART_STOP_BITS
		.stopbits = CONF_UART_STOP_BITS,
#endif
	};

	/* Configure console UART. */
	sysclk_enable_peripheral_clock(CONSOLE_UART_ID);
	stdio_serial_init(CONF_UART, &uart_serial_options);
}



	
static void TC0_CH0_Init(void)
{
	float temp_rc;
	uint32_t ra, rc;

	/* Configure the PMC to enable the TC module. */
	sysclk_enable_peripheral_clock(ID_TC0);

	/* Init TC to waveform mode. */
	tc_init(TC0,
			0,
			/* Waveform Clock Selection */
			s_tc0_ch0_waveconfig.clock_sel	/* MCK / 2 */
			| TC_CMR_WAVE /* Waveform mode is enabled */
			| TC_CMR_ACPA_SET /* RA Compare Effect: set */
			| TC_CMR_ACPC_CLEAR /* RC Compare Effect: clear */
			| TC_CMR_CPCTRG /* UP mode with automatic trigger on RC Compare */
	);

	/* Configure waveform frequency and duty cycle. */
	temp_rc = (sysclk_get_peripheral_bus_hz(TC0) / freq_divisors[s_tc0_ch0_waveconfig.clock_sel]) / (float)s_tc0_ch0_waveconfig.frequency;
	printf("temp_rc = %f\r\n", temp_rc);

	rc = temp_rc;

	if(temp_rc - (uint32_t)temp_rc >= 0.5)
		rc++;
		
	tc_write_rc(TC0, 0, rc);

	ra = (100 - s_tc0_ch0_waveconfig.dutycycle) * rc / 100;
	tc_write_ra(TC0, 0, ra);

	printf("Start waveform: Frequency = %d Hz, Duty Cycle = %2d%%\n\r", s_tc0_ch0_waveconfig.frequency, s_tc0_ch0_waveconfig.dutycycle);

	/* Enable TC TC_CHANNEL_WAVEFORM. */
	tc_start(TC0, 0);
	tc_enable_interrupt(TC0, 0, TC_IER_CPCS);
	NVIC_EnableIRQ(TC0_IRQn);
}




void TC0_Handler(void)
{
	if ((tc_get_status(TC0, 0) & TC_SR_CPCS) == TC_SR_CPCS)
	{
		//ioport_toggle_pin_level(LED0_GPIO);
		s_tc0_count++;
	}
}




static uint32_t TC0_GetCurrentCount(void)
{
	return s_tc0_count;
}




static uint32_t DHT_read(uint8_t *prh_i, uint8_t *prh_d, uint8_t *ptemp_i, uint8_t *ptemp_d)
{
	uint32_t dht_data = 0;
	uint8_t rh_i, rh_d, temp_i, temp_d;
	volatile uint8_t i, check_sum = 0;
	volatile uint32_t begin;


	//(1)
	// MCU sends out start signal and pulls down voltage for at least
	// 18ms to let DHT11detect this signal
	
	//ioport_set_pin_dir(PIN_DATA, IOPORT_DIR_OUTPUT);
	ioport_set_pin_level(PIN_DATA, 0);
	delay_ms(20);

	ioport_set_pin_dir(PIN_DATA, IOPORT_DIR_INPUT);
	//ioport_set_pin_mode(PIN_DATA, IOPORT_MODE_PULLUP);

	begin = TC0_GetCurrentCount();

	// Before (2), PIN_DATA checks whether DHT11 exists or not.
	while (ioport_get_pin_level(PIN_DATA) == 0)
	{
		// Means there is no DHT11...
		if(TC0_GetCurrentCount() - begin >= 60000)
		{
			printf("E1\r\n");
			return 1;
		}
	}

	begin = TC0_GetCurrentCount();

	//(2) MCU pulls up voltage and waits for DHT response (20 ~ 40usec)
	while (ioport_get_pin_level(PIN_DATA) == 1)
	{
		if(TC0_GetCurrentCount() - begin > 5000)	//50us
		{
			printf("E2\r\n");
			return 1;
		}
	}

	begin = TC0_GetCurrentCount();

	//(3)
	// DHT sends out response signal and keeps it for 80usec
	while (ioport_get_pin_level(PIN_DATA) == 0)
	{
		if(TC0_GetCurrentCount() - begin > 90)	//90us
		{
			printf("E3\r\n");
			return 1;
		}
	}

	begin = TC0_GetCurrentCount();

	//(4)
	// DHT pulls up voltage and keeps at for 80usec
	while (ioport_get_pin_level(PIN_DATA) == 1)
	{
		if(TC0_GetCurrentCount() - begin > 90)	//90us
		{
			printf("E4\r\n");
			return 1;
		}
	}

	/* Now start data transmission from DHT to MCU */
	for (i = 0; i < 32; i++)
	{
		while (ioport_get_pin_level(PIN_DATA) == 0);	// 50us

		begin = TC0_GetCurrentCount();

		while (ioport_get_pin_level(PIN_DATA) == 1);

		if(TC0_GetCurrentCount() - begin >= 50)	//50us
			dht_data |= (0x80000000 >> i);
	}

	for (i = 0; i < 8; i++)
	{
		while (ioport_get_pin_level(PIN_DATA) == 0); // 50us

		begin = TC0_GetCurrentCount();

		while (ioport_get_pin_level(PIN_DATA) == 1);

		if(TC0_GetCurrentCount() - begin >= 50)	//50us
			check_sum |= (0x80 >> i);
	}

	while (ioport_get_pin_level(PIN_DATA) == 0); // 50us

	delay_ms(10);

	ioport_set_pin_dir(PIN_DATA, IOPORT_DIR_OUTPUT);
	ioport_set_pin_level(PIN_DATA, 1);

	rh_i = dht_data >> 24;
	rh_d = (dht_data >> 16) & 0xFF;
	temp_i = (dht_data >> 8) & 0xFF;
	temp_d = dht_data & 0xFF;

	if ((rh_i + rh_d + temp_i + temp_d) == check_sum)
	{
		*prh_i =rh_i;
		*prh_d = rh_d;
		*ptemp_i = temp_i;
		*ptemp_d = temp_d;

		return 0;
	}
	else
	{
		printf("check_sum Failed 0x%X  0x%X\r\n", (rh_i + rh_d + temp_i + temp_d), check_sum);
		return 1;
	}
}




int main (void)
{
	uint8_t rh_i, rh_d, temp_i, temp_d;


	/* Initialize the SAM system */
	sysclk_init();

	/* Initialize the board */
	board_init();

	/* TRIG pin configuration */
	ioport_set_pin_dir(PIN_DATA, IOPORT_DIR_OUTPUT);
	ioport_set_pin_level(PIN_DATA, 1);

	configure_console();

	printf("SAME70 + DHT11 Test\r\n");
	printf("CPU clock = %d Hz\r\n", sysclk_get_cpu_hz());
	printf("Main clock = %d Hz\r\n", sysclk_get_main_hz());
	printf("Peripheral clock = %d Hz\r\n", sysclk_get_peripheral_hz());

	delay_ms(500);

	TC0_CH0_Init();

	while (1)
	{
		if(DHT_read(&rh_i, &rh_d, &temp_i, &temp_d) == 0)
			printf("Humidity %d.%d  Temperature %d.%d\r\n", rh_i, rh_d, temp_i, temp_d);
		else
		{
			printf("Error\r\n");

			ioport_set_pin_dir(PIN_DATA, IOPORT_DIR_OUTPUT);
			ioport_set_pin_level(PIN_DATA, 1);
		}

		/* Be sure to wait min. response time(= 6sec from datasheet). */
		delay_ms(6000);
	}
}

/* End of File */
